<?php

namespace Drupal\social_auth_itsme;

use Drupal\Core\Language\LanguageManager;
use Drupal\social_auth\SocialAuthDataHandler;
use Nascom\ItsmeApiClient\Request\Transaction\CreateTransactionRequest;
use Nascom\ItsmeApiClient\Request\Transaction\Service;
use Nascom\ItsmeApiClient\Response\Transaction\Transaction;
use Nascom\ItsmeApiClient\Request\Status\RetrieveStatusRequest;
use Nascom\ItsmeApiClient\Response\Status\Status;
use Nascom\ItsmeApiClient\Response\Status\Locale;

/**
 * Contains all the logic for itsme login integration.
 */
class ItsmeAuthManager {

  /**
   * @var \Drupal\social_auth_itsme\Settings\ItsmeAuthSettingsInterface
   */
  protected $settings;

  /**
   * The service client.
   *
   * @var \Nascom\ItsmeApiClient\Http\ApiClient\ApiClient
   */
  protected $client;

  /**
   * The user returned by the provider.
   *
   * @var Nascom\ItsmeApiClient\Response\Status\Status
   */
  protected $user;

  /**
   * @var \Drupal\social_auth\SocialAuthDataHandler
   */
  protected $dataHandler;

  /**
   * @var \Drupal\Core\Language\LanguageManager
   */
  private $languageManager;

  /**
   * {@inheritdoc}
   */
  public function setClient($client) {
    $this->client = $client;
    return $this;
  }

  /**
   * @param \Drupal\social_auth_itsme\Settings\ItsmeAuthSettings $settings
   *
   * @return $this
   */
  public function setSettings($settings) {
    $this->settings = $settings;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getClient() {
    return $this->client;
  }

  /**
   * ItsmeAuthManager constructor.
   *
   * @param \Drupal\social_auth\SocialAuthDataHandler $dataHandler
   * @param \Drupal\Core\Language\LanguageManager $languageManager
   */
  public function __construct(SocialAuthDataHandler $dataHandler, LanguageManager $languageManager) {
    $this->dataHandler = $dataHandler;
    $this->languageManager = $languageManager;
  }

  /**
   * {@inheritdoc}
   */
  public function getUserInfo() {
    if (!$this->user) {
      try {
        $retrieveStatusRequest = new RetrieveStatusRequest($this->getAccessToken());
        /** @var Status $status */
        $this->user = $this->client->handle($retrieveStatusRequest);
      }
      catch (\Exception $e) {
        return FALSE;
      }

      // Failed to authenticate
      if ($this->user->getStatus() !== Status::SUCCESS) {
        return FALSE;
      }
    }
    return $this->user;
  }

  /**
   * {@inheritdoc}
   */
  public function getAuthorizationUrl() {
    // Create a transaction with the itsme API
    $createTransactionRequest = new CreateTransactionRequest(
      $this->settings->getToken(),
      Service::LOGIN, // @TODO Hardcoded for now
      $GLOBALS['base_url'] . '/user/login/itsme/callback'
    );

    // Add scopes it provided
    if ($scopes = $this->settings->getScopes()) {
      $createTransactionRequest->setScopes($scopes);
    }

    // Load language but
    $language_id = $this->languageManager->getCurrentLanguage()->getId();
    \Drupal::moduleHandler()->alter('social_auth_itsme_locale', $language_id);

    // Add locale to request
    if (in_array($language_id, [Locale::EN, Locale::NL, Locale::FR, Locale::DE])) {
      $createTransactionRequest->setLocale();
    }

    try {
      /** @var Transaction $transaction */
      $transaction = $this->client->handle($createTransactionRequest);
      $this->dataHandler->set('token', $transaction->getToken());

      return $transaction->getAuthenticationUrl();
    }
    catch (\Exception $e) {
      return FALSE;
    }
  }

  /**
   * @return mixed
   */
  public function getAccessToken() {
    return $this->dataHandler->get('token');
  }

}
